// Copyright (c) Microsoft Corporation and Contributors.
// Licensed under the MIT License.

using AliGenieAssistantWinUI3.WindowExtensions;
using Microsoft.UI;
using Microsoft.UI.Composition.SystemBackdrops;
using Microsoft.UI.Windowing;
using Microsoft.UI.Xaml;
using Microsoft.UI.Xaml.Controls;
using System;
using System.Linq;
using System.Runtime.InteropServices;
using Windows.ApplicationModel.Core;
using Windows.ApplicationModel.Resources;
using Windows.Graphics;
using Windows.UI.Popups;
using Windows.UI.ViewManagement;
using WinRT;
using WinRT.Interop;

// To learn more about WinUI, the WinUI project structure,
// and more about our project templates, see: http://aka.ms/winui-project-info.

namespace AliGenieAssistantWinUI3
{
    /// <summary>
    /// An empty window that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainWindow : DesktopWindow
    {
        WindowsSystemDispatcherQueueHelper m_wsdqHelper; // See below for implementation.
        MicaController m_backdropController;
        SystemBackdropConfiguration m_configurationSource;
        AppWindowTitleBar titleBar;

        public MainWindow()
        {
            this.InitializeComponent();
            this.configTitleBar();

            Title = "è";
            Icon = "Assets/icon.ico";
            TrySetSystemBackdrop();
            this.Closing += MainWindow_Closing;
            // this.navbar.SelectedItem = navbar.MenuItems.ElementAt(0);
            InitCommunication();
            _ = DispatcherQueue.TryEnqueue(() =>
            {
                Hide();
            });
        }

        private void configTitleBar()
        {
            // ȡǰھ
            var _windowHandle = WindowNative.GetWindowHandle(this);
            var windowId = Win32Interop.GetWindowIdFromWindow(_windowHandle);

            // ȡӦôڶ
            var appWindow = AppWindow.GetFromWindowId(windowId);
            titleBar = appWindow.TitleBar;
            titleBar.ExtendsContentIntoTitleBar = true;
            titleBar.ButtonBackgroundColor = Colors.Transparent;
            titleBar.ButtonInactiveBackgroundColor = Colors.Transparent;
            //϶
            int windowWidth = this.Width;
            RectInt32 dragRect = new RectInt32();
            dragRect.X = 200;
            dragRect.Y = 0;
            dragRect.Width = windowWidth - 200;
            dragRect.Height = titleBar.Height;
            titleBar.SetDragRectangles(new RectInt32[]{ dragRect});
        }

        private void InitCommunication()
        {
            MQTTHelper helper = MQTTHelper.Instance();
            helper.Start();
        }

        private void MainWindow_Closing(object sender, WindowClosingEventArgs e)
        {
            Hide();
        }

        private async void NavigationView_SelectionChanged(NavigationView sender, NavigationViewSelectionChangedEventArgs args)
        {
            
            var selectedItem = (NavigationViewItem)args.SelectedItem;
            switch ((string)selectedItem.Tag)
            {
                case "Exit":
                    ResourceLoader resourceLoader = ResourceLoader.GetForViewIndependentUse();
                    ContentDialog contentDialog = new ContentDialog();
                    contentDialog.Content = resourceLoader.GetString("DialogContent");
                    contentDialog.XamlRoot = this.Content.XamlRoot;
                    contentDialog.PrimaryButtonText = resourceLoader.GetString("Yes");
                    contentDialog.SecondaryButtonText = resourceLoader.GetString("Min");
                    contentDialog.CloseButtonText = resourceLoader.GetString("No");
                    contentDialog.IsPrimaryButtonEnabled = true;
                    var r = await contentDialog.ShowAsync();
                    if (r == ContentDialogResult.Primary)
                    {
                        // Cancel the close event 
                        App.Current.Exit();
                    }
                    else if (r == ContentDialogResult.Secondary)
                    {
                        Hide();
                    }
                    this.navbar.SelectedItem = navbar.MenuItems.ElementAt(0);
                    break;
                case "Log":
                    contentFrame.Navigate(typeof(LogPage));
                    break;
                case "Setting":
                    contentFrame.Navigate(typeof(SettingPage));
                    break;
                case "About":
                    contentFrame.Navigate(typeof(AboutPage));
                    break;
                default:
                    break;
            }           
            
        }

        bool TrySetSystemBackdrop()
        {
            if (MicaController.IsSupported())
            {
                m_wsdqHelper = new WindowsSystemDispatcherQueueHelper();
                m_wsdqHelper.EnsureWindowsSystemDispatcherQueueController();

                // Create the policy object.
                m_configurationSource = new SystemBackdropConfiguration();
                this.Activated += Window_Activated;
                this.Closed += Window_Closed;
                ((FrameworkElement)this.Content).ActualThemeChanged += Window_ThemeChanged;

                // Initial configuration state.
                m_configurationSource.IsInputActive = true;
                SetConfigurationSourceTheme();

                m_backdropController = new MicaController();
    
                // Enable the system backdrop.
                // Note: Be sure to have "using WinRT;" to support the Window.As<...>() call.
                m_backdropController.AddSystemBackdropTarget(this.As<Microsoft.UI.Composition.ICompositionSupportsSystemBackdrop>());
                m_backdropController.SetSystemBackdropConfiguration(m_configurationSource);
                return true; // succeeded
            }

            return false; // Mica is not supported on this system
        }

        private void Window_Activated(object sender, WindowActivatedEventArgs args)
        {
            m_configurationSource.IsInputActive = args.WindowActivationState != WindowActivationState.Deactivated;
        }

        private void Window_Closed(object sender, WindowEventArgs args)
        {
            // Make sure any Mica/Acrylic controller is disposed
            // so it doesn't try to use this closed window.
            if (m_backdropController != null)
            {
                m_backdropController.Dispose();
                m_backdropController = null;
            }
            this.Activated -= Window_Activated;
            m_configurationSource = null;
        }

        private void Window_ThemeChanged(FrameworkElement sender, object args)
        {
            if (m_configurationSource != null)
            {
                SetConfigurationSourceTheme();
            }
        }

        private void SetConfigurationSourceTheme()
        {
            switch (((FrameworkElement)this.Content).ActualTheme)
            {
                case ElementTheme.Dark: 
                    m_configurationSource.Theme = Microsoft.UI.Composition.SystemBackdrops.SystemBackdropTheme.Dark;
                    titleBar.ButtonForegroundColor = Colors.White;
                    break;
                case ElementTheme.Light: 
                    m_configurationSource.Theme = Microsoft.UI.Composition.SystemBackdrops.SystemBackdropTheme.Light;
                    titleBar.ButtonForegroundColor = Colors.Black;
                    break;
                case ElementTheme.Default: 
                    m_configurationSource.Theme = Microsoft.UI.Composition.SystemBackdrops.SystemBackdropTheme.Default; 
                    break;
            }
        }

        private void navbar_Loaded(object sender, RoutedEventArgs e)
        {
            navbar.SelectedItem = navbar.MenuItems.ElementAt(0);
        }
    }

    internal class WindowsSystemDispatcherQueueHelper
    {
        [StructLayout(LayoutKind.Sequential)]
        struct DispatcherQueueOptions
        {
            internal int dwSize;
            internal int threadType;
            internal int apartmentType;
        }

        [DllImport("CoreMessaging.dll")]
        private static extern int CreateDispatcherQueueController([In] DispatcherQueueOptions options, [In, Out, MarshalAs(UnmanagedType.IUnknown)] ref object dispatcherQueueController);

        object m_dispatcherQueueController = null;
        public void EnsureWindowsSystemDispatcherQueueController()
        {
            if (Windows.System.DispatcherQueue.GetForCurrentThread() != null)
            {
                // one already exists, so we'll just use it.
                return;
            }

            if (m_dispatcherQueueController == null)
            {
                DispatcherQueueOptions options;
                options.dwSize = Marshal.SizeOf(typeof(DispatcherQueueOptions));
                options.threadType = 2;    // DQTYPE_THREAD_CURRENT
                options.apartmentType = 2; // DQTAT_COM_STA

                CreateDispatcherQueueController(options, ref m_dispatcherQueueController);
            }
        }
    }
}
